function X_hat = extrapolation(X, t_start, t_end, dt, R, alpha, loc, shape, gamma, delta, forecast_horizon, variant, clean)
    % Performs the extrapolation
    
    % Frchet CDF and PDF
    F = @(x) exp(-((x-loc)/shape)^(-alpha));
    f = @(x) (alpha/shape)*((x-loc)/shape)^(-alpha-1)*exp(-((x-loc)/shape)^(-alpha));
        
    time = t_start:dt:t_end;
    T = length(time);

    % Initialize forecast sample
    forecast_sample = X(T-R: T);
    
    % Get learning samples and the values corresponding to forecast
    % horizon
    [learning_samples, observed_values] = get_learning_samples(X, T, R, length(forecast_sample), forecast_horizon);

    % Initial guess, tunable, must be same size as forecast sample
    lambda0 = (0.7:0.3/R:1)';
    if length(lambda0)~= length(forecast_sample)
        error('Initial guess lambda0 must be the same size as forecast sample!')
    end
    % Calculate lambda and max-stable predictor
    lambda_end = gradient_descent(lambda0, learning_samples, observed_values, F, f, gamma, delta, variant, clean);
    X_hat = get_extrapolator(forecast_sample.', lambda_end);
end

