function [dQ, non_differentiable] = get_gradient(idx, learning_samples, observed_values, X_tilde, X_tilde_hat, lambda, F, f, gamma)
 % Computes the gradient of Q_j of Phi 
 
    % X_h is the observation that corresponds to forecast within the
    % next learning sample position-wise
    X_h = observed_values(idx, 1);
    % Initalize learning sample
    sub_sample = learning_samples(idx, :).';
    % Calculate predictor
    X_hat_h = get_extrapolator (sub_sample, lambda);
    
    % Define corresponding X_tilde values (from bootstrap samples)
    X_tilde_h = X_tilde(idx, :)';
    X_tilde_hat_h = X_tilde_hat(idx);

    % Define Gradient
    dQ = zeros(size(lambda, 1),1);
    non_differentiable = 0;
    for j = 1:size(lambda, 1)
        % Check if Phi is differentiable. This is the case if there is
        % no turnover/kink in the maximum functions
        if X_tilde_hat_h == X_hat_h || X_h==lambda(j)*sub_sample(j) || sum((sub_sample.*lambda)==X_hat_h)>1
            % Kink occurs, Phi is not differentiable here
            non_differentiable = 1;
        else
            dq = (2*(X_h<lambda(j)*sub_sample(j)) - 1 - gamma*(X_tilde_hat_h<X_hat_h)...
            +2*gamma*F(lambda(j)*sub_sample(j)))*f(lambda(j)*sub_sample(j))...
            *sub_sample(j)*(lambda(j)*sub_sample(j) == X_hat_h);

            dp = gamma*(X_tilde_hat_h>X_hat_h)*f(X_tilde_hat_h)*X_tilde_h(j)...
            *(lambda(j)*X_tilde_h(j) == X_tilde_hat_h);

            dQ(j,1) = dq-dp;
        end
    end
end

