function [dQ, non_differentiable] = get_gradient_variant(idx, learning_samples, observed_values, lambda, F, f, gamma)
    % Computes the gradient of Q_j of alternative formulation of Phi
    N = size(learning_samples, 1);
    
    % X_h is the observation that corresponds to forecast 
    X_h = observed_values(idx, 1);
    % Initialize learning sample
    sub_sample = learning_samples(idx, :)';
    % Calculate predictor
    X_hat_h = max(sub_sample.*lambda);

    % Define Gradient
    dQ = ones(size(lambda, 1),1);
    non_differentiable = 0;

    for j = 1:size(lambda, 1)
        % Always check if Phi is differentiable. This is the case if there is
        % no turnover/kink in the maximum functions
        X_hat_sum_j = 0;
        X_hat_sum_i = 0;
        for i = 1:idx-1
            sub_sample_i = learning_samples(i,:)';
            X_hat_sum_j = X_hat_sum_j + f(lambda(j)*sub_sample(j))*sub_sample(j)*(max(lambda.*sub_sample_i) < max(lambda.*sub_sample)); 
            X_hat_sum_i = X_hat_sum_i + f(lambda(j)*sub_sample_i(j))*sub_sample_i(j)*(max(lambda.*sub_sample_i) > max(lambda.*sub_sample))...
                *(lambda(j)*sub_sample_i(j) == X_hat_h);
            if max(lambda.*sub_sample_i) == max(lambda.*sub_sample)
                % Kink occurs, Phi is not differentiable here 
                non_differentiable = 1;
            end
        end

        if  X_h==lambda(j)*sub_sample(j) || sum((sub_sample.*lambda)==X_hat_h)>1
            % Kink occurs, Phi is not differentiable here
            dQ(j,1) = 0;
            non_differentiable = 1;
        else
            dQ(j,1) = (f(lambda(j)*sub_sample(j))*sub_sample(j)*(2*(X_h<lambda(j)*sub_sample(j))...
            - 1 + gamma/N - 2*gamma*F(lambda(j)*sub_sample(j))) + (2*gamma/N)*X_hat_sum_j)...
            *(lambda(j)*sub_sample(j) == X_hat_h) + (2*gamma/N)*X_hat_sum_i;

        end
    end
end

