function [learning_samples, observed_values] = get_learning_samples(X, T, R, len, forecast_horizon)
    % Creates and returns learning samples
    
    % Amount of learning samples; by default takes as much available data
    % as possible
    ls_count = floor((T-R-1)/R);

    % Store learning samples row-wise in a matrix and the observed values
    % corresponding to forecast horizon in a vector
    learning_samples = zeros(ls_count, len);
    observed_values = zeros(ls_count, 1);
    
    % Locations/time stamps of the learning samples
    loc = T-2*R:-R:1;
    
    % Initialize learning samples with values from X
    for h = 1:ls_count
        learning_samples(h,:) = X(loc(h):loc(h)+len-1);
        observed_values(h,1) = X(loc(h)+len-1+forecast_horizon);
    end
end


