function lambda_end = gradient_descent(lambda_start, learning_samples, observed_values, F, f, gamma, delta, variant, clean)
    % Performs stochastic gradient descent for the function Phi
    
    % Initial step size for harmonic series (1/n) is 1/1 
    step_size_divisor = 1;
    
    % Initialize final lambda
    lambda_end = lambda_start;
    % lambda variable to store values of each iteration
    lambda = lambda_start;
    % lambda of previous iteration to check the termination condition
    lambda_prev = 0;
    
    % Generate Bootstrap samples 
    X_tilde = zeros(size(learning_samples,1), size(learning_samples,2));
    X_tilde_hat = zeros(1, size(learning_samples, 1));
    Y = zeros(1, size(learning_samples, 1));
    for i = 1:size(X_tilde,1)
        for j = 1:size(X_tilde, 2)
            X_tilde(i,j) = learning_samples(randi(numel(learning_samples)));
        end
        X_tilde_hat(1,i) = get_extrapolator(X_tilde(i, :)', lambda);
        Y(1,i) = F(X_tilde_hat(1,i));
    end
    
    while norm(lambda-lambda_prev) > delta

        % Choose learning sample randomly
        hj = randi(size(learning_samples,1));
        
        % Compute Gradient
        if ~ variant
            [dQ, non_differentiable] = get_gradient(hj, learning_samples, observed_values, X_tilde, X_tilde_hat, lambda, F, f, gamma);
        else
            [dQ, non_differentiable] = get_gradient_variant(hj, learning_samples, observed_values, lambda, F, f, gamma);
        end
        
        % If clean is enabled then check for the cleanness of the
        % gradients, else just for non differentiablity
        if clean 
            if non_differentiable || check_for_cleanness(hj, learning_samples, observed_values, X_tilde, X_tilde_hat, lambda, F, f, gamma, variant)
                continue
            end
        else
            if non_differentiable
                continue
            end
        end
        
        % Stochastic Gradient Descent
        lambda_prev = lambda;
        lambda = max(0.000001, lambda - (1/step_size_divisor)*dQ);
        
        % Adjust step size, according to harmonic series (1/n)
        step_size_divisor = step_size_divisor+1;

        % Check if new lambda makes any improvement of the solution. If so,
        % store it.
        if ~variant
            if phi(lambda_end, learning_samples, observed_values, Y, F, gamma) > phi(lambda, learning_samples, observed_values, Y, F, gamma)
                lambda_end = lambda;
            end
        else 
            if phi_variant(lambda_end, learning_samples, observed_values, F, gamma) > phi_variant(lambda, learning_samples, observed_values, F, gamma)
                lambda_end = lambda;
            end
        end
    end 
end

